from datetime import timedelta 
from django.urls import reverse
from django.utils import timezone
from decimal import Decimal
from django.shortcuts import render, redirect, get_object_or_404
from django.http import HttpResponse, HttpResponseForbidden
from django.contrib.auth.decorators import login_required
from django.contrib import messages
from django.views.decorators.http import require_POST
import qrcode
from payments.models import Payment
from products.models import Product
from rest_framework import status
from rest_framework.response import Response
from rest_framework.decorators import api_view, permission_classes

from quotations import qr_generator
from quotations.serializer import InquirySerializer
from .models import Inquiry, Quotation , CustomerQuoteInfo
from customers.models import Customer
from account.models import Account
from config.apipermissions import AllowAllPermissions 
from django.core.files import File
from io import BytesIO

from uom.models import UnitOfMeasure
from fundis.models import Fundi
from colors.models import Color 
from gauge.models import Gauge

from django.core.paginator import Paginator
from django.http import HttpResponse
import csv
from django.core.exceptions import ObjectDoesNotExist

from rest_framework import status
from rest_framework.response import Response

@login_required
def quotations(request):
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_sales:
        if request.user.is_admin_user or request.user.is_super_user:
            quotations = Quotation.objects.all().order_by('-created_at')  # For admin and super admin, get all quotations
        else:
            # For other users (sales), filter quotations where the seller is the current user
            quotations = Quotation.objects.filter(seller=request.user).order_by('-created_at')
        
        # Apply filters
        code = request.GET.get('code')
        if code:
            quotations = quotations.filter(qid__icontains=code)
        
        status = request.GET.get('status')
        if status:
            quotations = quotations.filter(status__icontains=status)
        
        customer_name = request.GET.get('customer_name')
        if customer_name:
            quotations = quotations.filter(customer_info__customer_name__icontains=customer_name)
        
        from_date = request.GET.get('from_date')
        to_date = request.GET.get('to_date')
        if from_date and to_date:
            quotations = quotations.filter(created_at__range=[from_date, to_date])
        
        # Pagination
        paginator = Paginator(quotations, 10)  # Show 10 quotations per page
        page_number = request.GET.get('page')
        page_obj = paginator.get_page(page_number)
        
        data = {
            "page_obj": page_obj,
        }
        
        if 'export' in request.GET and request.GET['export'] == 'true':
            response = HttpResponse(content_type='text/csv')
            response['Content-Disposition'] = 'attachment; filename="quotations.csv"'
            
            writer = csv.writer(response)
            writer.writerow(['QUOTATION ID', 'Customer Name', 'Customer Mobile',  'Fundi Name', 'Fundi Number', 'Delivery Location', 'Delivery Mode', 'Payment Terms', 'Lead Source', 'Seller', 'Total Amount', 'Status', 'Created At'])

            from django.core.exceptions import ObjectDoesNotExist

            for quotation in quotations:
                try:
                    customer_quote_info = CustomerQuoteInfo.objects.get(quotation=quotation)
                    customer_name = customer_quote_info.customer.customer_name if customer_quote_info.customer else ""
                    customer_mobile = customer_quote_info.customer.customer_mobile if customer_quote_info.customer else ""
                    fundi = customer_quote_info.customer.fundi if (customer_quote_info.customer and customer_quote_info.customer.fundi) else ""
                    fundi_details = customer_quote_info.customer.fundi_details if (customer_quote_info.customer and customer_quote_info.customer.fundi_details) else ""
                except ObjectDoesNotExist:
                    # If CustomerQuoteInfo does not exist, set its info and customer's info to ""
                    customer_quote_info = CustomerQuoteInfo()
                    customer_name = ""
                    customer_mobile = ""
                    fundi = ""
                    fundi_details = ""

                writer.writerow([
                    quotation.qid, 
                    customer_name,
                    customer_mobile, 
                    fundi,
                    fundi_details,
                    customer_quote_info.delivery_location if customer_quote_info.delivery_location else "",  
                    customer_quote_info.delivery_mode if customer_quote_info.delivery_mode else "",   
                    customer_quote_info.payment_terms if customer_quote_info.payment_terms else "",   
                    customer_quote_info.lead_source if customer_quote_info.lead_source else "",  
                    customer_quote_info.seller.email if customer_quote_info.seller else "",   
                    quotation.total_amount, 
                    quotation.status, 
                    quotation.created_at
                ])

            return response
    
        return render(request, "sales/quotation/quotations.html", data)
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

@login_required
def add_quotation(request):
    colors = Color.objects.all()
    products = Product.objects.all() 
    gauges = Gauge.objects.all()
    data = {
        "products" : products,
        "colors": colors,
        "gauges":gauges
    }
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_sales: 
        return render(request, "sales/quotation/index.html", data)
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")
    
@login_required
def quotation_detail(request, quotation_id):
    colors = Color.objects.all()
    gauges = Gauge.objects.all()
    uoms = UnitOfMeasure.objects.all()
    products = Product.objects.all()

    if request.user.is_admin_user or request.user.is_super_user or request.user.is_sales:
        # Get the quotation object
        quotation = get_object_or_404(Quotation, id=quotation_id) 
        remainingAmount = quotation.total_amount - quotation.amount_paid
        print(remainingAmount)
        # Get related customer quote info
        customer_quote_info = CustomerQuoteInfo.objects.filter(quotation=quotation).first()
        
        # Get related inquiries
        inquiries = Inquiry.objects.filter(quotation=quotation)

        # Get related payments 
        payments = Payment.objects.filter(quotation=quotation).order_by('-created_at')
        paginator = Paginator(payments, 3)  # Show 3 payments per page
        page_number = request.GET.get('page')
        page_obj = paginator.get_page(page_number)
        
        total_pages = paginator.num_pages  # Calculate the total number of pages
        
        data = {
            "colors" : colors, 
            "gauges" : gauges,
            "uoms" : uoms ,
            "products" : products,
            "quotation": quotation,
            "customer_quote_info": customer_quote_info,
            "inquiries": inquiries,
            "payments": page_obj,  # Pass the paginated payments to the template
            "total_pages": total_pages,  # Pass the total number of pages to the template
            "remainingAmount" : remainingAmount
        }
        
        
        return render(request, "sales/quotation/quotation_detail.html", data)
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

@require_POST
@login_required
def delete_quotation(request, quotation_id):
    if request.user.is_admin_user or request.user.is_super_user:
        try:
            quotation = get_object_or_404(Quotation, id=quotation_id)
            quotation.delete()
            messages.success(request, 'Quotation deleted successfully.')
            return redirect(reverse('sales:quotations'))
        except Exception as e:
            messages.error(request, str(e))
            return redirect(reverse('sales:quotations'))
    else:
        messages.error(request, 'You are not authorized to delete this quotation.')
        return redirect(reverse('sales:quotations'))


@api_view(['POST'])
@permission_classes([AllowAllPermissions])  # Adjust permissions as needed
def create_quotation(request):
    """
    API endpoint to receive and save quotation data.
    """
    if request.method == 'POST':
        try: 
            customer_data = request.data.get('customerInfo')
            inquiry_data = request.data.get('inquiries')
            seller_id = request.data.get('seller_id')
            total_amount = request.data.get('total_amount')
            securityKey = request.data.get('securityKey')

            # Security check (consider using a custom permission class)
            if securityKey != "MABATIIQSECURITY":
                return Response({'error': 'Invalid security key. Access denied.'}, status=status.HTTP_401_UNAUTHORIZED)

            # Get seller (handle potential errors)
            try:
                seller = Account.objects.get(pk=seller_id)  # Use primary key (pk) for better performance
            except Account.DoesNotExist:
                return Response({'error': 'Invalid seller ID.'}, status=status.HTTP_400_BAD_REQUEST)

            # Permission check (handle potential errors)
            if not (seller.is_superuser or seller.is_admin_user or seller.is_seller or seller.is_pos):
                return Response({'error': 'Permission denied. You need admin, seller, or POS privileges to create quotations.'}, status=status.HTTP_403_FORBIDDEN)
            
            # Check for existing customer using phone number
            customer_phone = customer_data['customerMobile']
            existing_customer = Customer.objects.filter(customer_mobile=customer_phone).first()

            # Check for existing fundi using fundi details
            fundi_details = customer_data['fundiNumber']
            existing_fundi = Fundi.objects.filter(fundi_details=fundi_details).first()

            if existing_customer and existing_fundi: 
                customer_info = existing_customer
            elif existing_customer:
                # Customer exists, use existing customer and create new fundi
                customer_info = existing_customer
                Fundi.objects.create(
                    fundi=customer_data['fundiName'],
                    fundi_details=customer_data['fundiNumber'],
                    added_by=seller,
                )
            elif existing_fundi:
                # Fundi exists, create new customer and use existing fundi
                customer_info = Customer.objects.create(
                    customer_name=customer_data['customerName'],
                    customer_mobile=customer_data['customerMobile'],
                    fundi=customer_data['fundiName'],  # Associate with existing fundi
                    added_by=seller,
                )
            else:
                # Neither customer nor fundi exist, create both
                try:
                    customer_info = Customer.objects.create(
                        customer_name=customer_data['customerName'],
                        customer_mobile=customer_data['customerMobile'],
                        fundi=customer_data['fundiName'],
                        fundi_details=customer_data['fundiNumber'],
                        added_by=seller,
                    )
                except Exception as e: 
                    print(f"Error creating Customer: {e}")

                # Create Fundi object (if needed, handle potential errors)
                try:
                    Fundi.objects.create(
                        fundi=customer_data['fundiName'],
                        fundi_details=customer_data['fundiNumber'],
                        added_by=seller,
                    )
                except Exception as e:  # Catch potential exceptions during Fundi creation
                    print(f"Error creating Fundi: {e}")
                    # Consider logging the error for further analysis
 

            # Create Quotation object
            quotation = Quotation.objects.create(
                customer_info=customer_info,
                seller=seller,
                total_amount=total_amount,
                #qr_code =  qr_generator(quotation.qid,customer_data) 
            ) 

             
 
            

            # Create CustomerQuoteInfo object
            customer_quote_info = CustomerQuoteInfo.objects.create(
                customer=customer_info,
                quotation=quotation,
                seller=seller,
                delivery_location=customer_data['deliveryLocation'],
                delivery_mode=customer_data['deliveryMode'],
                payment_terms=customer_data['paymentTerms'],
                lead_source=customer_data['leadSource'],
            )

            # Create Inquiry objects for each item
            for inquiry in inquiry_data:
                Inquiry.objects.create(
                    quotation=quotation,
                    seller=seller,
                    product_id=inquiry['product'],
                    color=Color.objects.filter(id=inquiry['color']).first() if inquiry['color'] != 'N/A' else None,
                    gauge=Gauge.objects.filter(id=inquiry['gauge']).first() if inquiry['gauge'] != 'N/A' else None,
                    size=inquiry['size'] if inquiry['size'] != 'N/A' else None,
                    uom=UnitOfMeasure.objects.filter(unit=inquiry['uom']).first() if inquiry['uom'] != 'N/A' else None,
                    quantity=inquiry['quantity'],
                    unit_price=inquiry['unit_price'],
                    amount=inquiry['amount'],
                )
                qr_data = {
                    "QUOTATION CODE": quotation.qid,
                    "SELLER" : quotation.seller.email,  
                    "AMOUNT": quotation.total_amount,
                    "CUSTOMER INFO": customer_data['customerName'], 
                    "VALIDITY" : timezone.now() + timedelta(days=30),
                    "STATUS" : "PENDING"
                } 
                success, error = qr_generator.generate_qr_code(quotation.id, qr_data)
                if success:
                    print("QR code saved successfully.")
                else:
                    print(f"Error saving QR code: {error}")
                

            return Response({'message': 'Quotation created successfully!', 'quotation_id': quotation.id}, status=status.HTTP_201_CREATED)

        except Exception as e:
            print(f"Error creating quotation: {e}")
            return Response({'error': 'An error occurred while creating quotation.'}, status=status.HTTP_400_BAD_REQUEST)

    else:
        return Response({'error': 'Invalid request method. Use POST.'}, status=status.HTTP_405_METHOD_NOT_ALLOWED)

# views.py

from django.http import JsonResponse
from django.views.decorators.csrf import csrf_exempt
from .models import Inquiry, Account, Quotation
import json

from django.http import JsonResponse
from django.views.decorators.csrf import csrf_exempt
from .models import Inquiry, Account, Quotation
import json
import re
@csrf_exempt
def update_inquiry(request):
    if request.method == 'POST':
        try:
            # Parse the JSON data from the request body
            data = json.loads(request.body)
            print("Received data:", data)
            
            inquiry_id = data.get('inquiryId')  # Ensure this matches the key in your JSON data
            if not inquiry_id:
                return JsonResponse({'error': 'Inquiry ID is required'}, status=400)
            print("Inquiry ID:", inquiry_id)
            
            # Fetch the inquiry object from the database
            inquiry = Inquiry.objects.filter(id=inquiry_id).first()
            if not inquiry:
                return JsonResponse({'error': 'Inquiry not found'}, status=404)
            print("Found inquiry:", inquiry)

            # Calculate the old total amount for the inquiry
            old_amount = inquiry.amount if inquiry.amount else 0
            print("Old amount:", old_amount)

            # Update the inquiry fields
            admin_id = data.get('adminId')
            if admin_id:
                inquiry.updated_by = Account.objects.filter(id=admin_id).first()
            productCode = data.get('product')
            print("PID",productCode)
            if productCode:
                inquiry.product = Product.objects.filter(product_id=productCode).first()
            inquiry.quantity = data.get('quantity', inquiry.quantity)
            # Update gauge
            gauge_id = data.get('gauge')
            if gauge_id:
                inquiry.gauge = None if gauge_id == "N/A" else Gauge.objects.filter(id=gauge_id).first() 

            # Update color
            color_id = data.get('color')
            if color_id:
                inquiry.color = None if color_id == "N/A" else Color.objects.filter(id=color_id).first()

            inquiry.size = data.get('size', inquiry.size)

            # Update UoM
            uom_id = data.get('uom')
            if uom_id:
                inquiry.uom = None if uom_id == "N/A" else UnitOfMeasure.objects.filter(id=uom_id).first()

 
            inquiry.size = data.get('size', inquiry.size) 
            inquiry.unit_price = data.get('unitPrice', inquiry.unit_price)

            # Calculate the new total amount for the inquiry
            quantity = inquiry.quantity or 0
            
            size = inquiry.size or 0
            unit_price = inquiry.unit_price or 0

            print(quantity)
            print(size)
            print(unit_price)
            new_amount = Decimal(quantity) * Decimal(size) * Decimal(unit_price)
            inquiry.amount = new_amount
            print("New amount:", new_amount)

            # Update the quotation's total amount
            quotation = inquiry.quotation
            if quotation.amount_paid is None:
                quotation.amount_paid = 0

            # Subtract the old amount and add the new amount
            quotation.total_amount = quotation.total_amount - old_amount + new_amount
            
            quotation.save()

            # Save the updated inquiry
            inquiry.save()

            return JsonResponse({'success': True})

        except KeyError as e:
            return JsonResponse({'error': f'Missing field: {str(e)}'}, status=400)
        except Exception as e:
            print(f"Error updating inquiry: {e}")
            return JsonResponse({'error': 'An error occurred while updating the inquiry.'}, status=500)
    else:
        return JsonResponse({'error': 'Method not allowed'}, status=405)



def delete_inquiry(request):
    if request.method == 'POST':
        try:
            # Parse the JSON data from the request body
            data = json.loads(request.body)
            inquiry_id = data.get('inquiryId')

            print(data)

            # Find the inquiry
            inquiry = Inquiry.objects.filter(id=inquiry_id).first()
            if not inquiry:
                return JsonResponse({'error': 'Inquiry not found'}, status=404)

            # Get the associated quotation
            quotation = inquiry.quotation
            if not quotation:
                return JsonResponse({'error': 'Quotation not found for the inquiry'}, status=404)

            # Deduct the amount from the quotation
            quotation.total_amount -= inquiry.amount
            quotation.save()

            # Check if there are other inquiries associated with the same quotation
            other_inquiries_count = Inquiry.objects.filter(quotation=quotation).exclude(id=inquiry_id).count()

            if other_inquiries_count == 0:
                # If it's the only inquiry associated with the quotation, delete the quotation
                quotation.delete()
                return JsonResponse({'success': True, 'message': 'Quotation deleted because it had only one inquiry',"redirect":"home"})

            # Delete the inquiry
            inquiry.delete()

            return JsonResponse({'success': True})

        except Exception as e:
            print(f"Error deleting inquiry: {e}")
            return JsonResponse({'error': 'An error occurred while deleting the inquiry.'}, status=500)
    else:
        return JsonResponse({'error': 'Method not allowed'}, status=405)

from django.http import JsonResponse
from .models import Inquiry, Quotation 
import json

def update_discount_inquiry(request):
    data = json.loads(request.body)
    print("Received data:", data)
    inquiry_id = data.get('inquiryId')
    discount_amount = data.get('discountAmount')
    request_user_id = data.get('requestUserId')

    # Check if the user is admin or super admin
    if request.user.is_admin_user or request.user.is_super_user:
        # If admin or super admin, affect the Inquiry amount, Quotation total amount, and validator
        discount_status = data.get('discountStatus')

        try:
            inquiry = Inquiry.objects.get(pk=inquiry_id)
            quotation = inquiry.quotation
        except Inquiry.DoesNotExist:
            return JsonResponse({'error': 'Inquiry not found'}, status=404)

        # Update Inquiry fields
        if discount_amount is not None:
            inquiry.discount_amount = discount_amount
        if discount_status is not None:
            inquiry.discount_status = discount_status

        # Update Quotation total amount
        if discount_amount is not None:
            
            discount_amount_unit_decimal = Decimal(discount_amount)
            disounted_total_inquiry_amount = discount_amount_unit_decimal * inquiry.size * inquiry.quantity
            inquiry.amount = disounted_total_inquiry_amount

            # Calculate the difference in discount amount
            previous_discount_amount = quotation.total_amount 
            diff_discount_amount =  previous_discount_amount - inquiry.amount
            # Update the Quotation total amount
            quotation.total_amount -= diff_discount_amount
            quotation.save() 
        # Update Inquiry validator
        inquiry.discount_validator = Account.objects.filter(id=request_user_id).first()
        inquiry.save()

    else:
        # If not admin or super admin, only update the Inquiry amount
        try:
            inquiry = Inquiry.objects.get(pk=inquiry_id)
        except Inquiry.DoesNotExist:
            return JsonResponse({'error': 'Inquiry not found'}, status=404)

        # Update Inquiry fields
        if discount_amount is not None:
            inquiry.discount_amount = discount_amount

        inquiry.save()

    serializer = InquirySerializer(inquiry)

    return JsonResponse({'message': 'Inquiry Discount updated successfully', 'data': serializer.data})

from django.http import JsonResponse
from rest_framework.decorators import api_view

@api_view(['GET'])
def check_customer_fundi_existence(request):
    """
    API to check if customer or fundi exists based on provided details.
    """
    if request.method == 'GET':
        customer_phone = request.GET.get('customer_phone')
        fundi_details = request.GET.get('fundi_details')

        # Check for existing customer using phone number
        existing_customer = Customer.objects.filter(customer_mobile=customer_phone).first()

        # Check for existing fundi using fundi details
        existing_fundi = Fundi.objects.filter(fundi_details=fundi_details).first()

        response_data = {}

        if existing_customer and existing_fundi:
            response_data['message'] = 'Both customer and fundi exist.'
            response_data['use_existing'] = True  # Use existing customer and fundi data
            response_data['customer_data'] = existing_customer.to_dict()  # Optional: Include customer data
            response_data['fundi_data'] = existing_fundi.to_dict()  # Optional: Include fundi data
        elif existing_customer:
            response_data['message'] = 'Customer exists.'
            response_data['use_existing'] = True  # Use existing customer data (fundi can be new)
            response_data['customer_data'] = existing_customer.to_dict()  # Optional: Include customer data
        elif existing_fundi:
            response_data['message'] = 'Fundi exists.'
            response_data['use_existing'] = True  # Use existing fundi data (customer can be new)
            response_data['fundi_data'] = existing_fundi.to_dict()  # Optional: Include fundi data
        else:
            response_data['message'] = 'Neither customer nor fundi exists.'
            response_data['use_existing'] = False  # Create new customer and/or fundi

        return JsonResponse(response_data)

    else:
        return JsonResponse({'error': 'Invalid request method. Use GET.'}, status=405)
    


